@@ -10,7 +10,7 @@
 ;; Maintainer: Le Wang
 
 ;; Created: Sat Jan  5 16:49:23 2013 (+0800)
-;; Version: 0.6
+;; Version: 0.7
 ;; Last-Updated:
 ;;           By:
 ;; URL: https://github.com/lewang/ws-butler
@@ -54,7 +54,7 @@
 ;;; Code:
 
 (eval-when-compile
-  (require 'cl))
+  (require 'cl-lib))
 
 (eval-and-compile
   (unless (fboundp 'setq-local)
@@ -80,7 +80,7 @@ i.e. only the \"virtual\" space is preserved in the buffer."
 
 (defcustom ws-butler-convert-leading-tabs-or-spaces
   nil
-  "Make leading whitespace be tabs or spaces
+  "Make leading whitespace be tabs or spaces.
 
 If `indent-tabs-mode' is non-nil, call `tabify', else call
 `untabify'. Do neither if `smart-tabs-mode' is enabled for this
@@ -92,7 +92,13 @@ changed in this specific way."
   :group 'ws-butler)
 
 (defcustom ws-butler-global-exempt-modes
-  '(markdown-mode)
+  '(special-mode
+    minibuffer-mode
+    comint-mode
+    term-mode
+    eshell-mode
+    diff-mode
+    markdown-mode)
   "Don't enable ws-butler in modes that inherit from these.
 
 This should be a list of trailing whitespace significant major-modes."
@@ -140,7 +146,7 @@ Also see `require-final-newline'."
      ;;
      ;; We refuse to remove final-newline regardless of the value of
      ;; `require-final-newline'
-     (when (looking-at "\n\\(\n\\|\\'\\)")
+     (when (looking-at-p "\n\\(?:\n\\|\\'\\)")
        (forward-char 1)))
    (when require-final-newline
      (unless (bolp)
@@ -149,7 +155,7 @@ Also see `require-final-newline'."
      (replace-match ""))))
 
 (defun ws-butler-maybe-trim-eob-lines (last-modified-pos)
-  "Delete extra newlines at end of buffer if LAST-MODIFIED-POS is in the patch of excess newlines."
+  "Trim newlines at EOB if LAST-MODIFIED-POS is inside the excess newlines."
   (interactive (list nil))
   (unless buffer-read-only
     (unless last-modified-pos
@@ -183,7 +189,7 @@ replaced by spaces, and vice versa if t."
      (when (and ws-butler-convert-leading-tabs-or-spaces
                 (not (bound-and-true-p smart-tabs-mode)))
        ;; convert leading tabs to spaces or v.v.
-       (let ((eol (point-at-eol)))
+       (let ((eol (line-end-position)))
          (if indent-tabs-mode
              (progn
                (skip-chars-forward "\t" eol)
@@ -242,11 +248,11 @@ ensure point doesn't jump due to white space trimming."
      (lambda (_prop beg end)
        (save-excursion
          (setq beg (progn (goto-char beg)
-                          (point-at-bol))
+                          (line-end-position))
                ;; Subtract one from end to overcome Emacs bug #17784, since we
                ;; always expand to end of line anyway, this should be OK.
                end (progn (goto-char (1- end))
-                          (point-at-eol))))
+                          (line-end-position))))
        (when (funcall ws-butler-trim-predicate beg end)
          (ws-butler-clean-region beg end))
        (setq last-end end)))
@@ -259,6 +265,8 @@ ensure point doesn't jump due to white space trimming."
                              (remove-list-of-text-properties start end '(ws-butler-chg))))))
 
 (defun ws-butler-after-change (beg end length-before)
+  "Update ws-butler text properties.
+See `after-change-functions' for explanation of BEG, END & LENGTH-BEFORE."
   (let ((type (if (and (= beg end) (> length-before 0))
                   'delete
                 'chg)))
@@ -303,22 +311,23 @@ for lines modified by you."
   :group 'ws-butler
   (if ws-butler-mode
       (progn
-        (add-hook 'after-change-functions 'ws-butler-after-change t t)
-        (add-hook 'before-save-hook 'ws-butler-before-save t t)
-        (add-hook 'after-save-hook 'ws-butler-after-save t t)
-        (add-hook 'before-revert-hook 'ws-butler-before-revert t t)
-        (add-hook 'after-revert-hook 'ws-butler-after-save t t)
-        (add-hook 'edit-server-done-hook 'ws-butler-before-save t t))
-    (remove-hook 'after-change-functions 'ws-butler-after-change t)
-    (remove-hook 'before-save-hook 'ws-butler-before-save t)
-    (remove-hook 'after-save-hook 'ws-butler-after-save t)
-    (remove-hook 'before-revert-hook 'ws-butler-before-revert t)
-    (remove-hook 'after-revert-hook 'ws-butler-after-save t)
-    (remove-hook 'edit-server-done-hook 'ws-butler-before-save t)))
+        (add-hook 'after-change-functions #'ws-butler-after-change t t)
+        (add-hook 'before-save-hook #'ws-butler-before-save t t)
+        (add-hook 'after-save-hook #'ws-butler-after-save t t)
+        (add-hook 'before-revert-hook #'ws-butler-before-revert t t)
+        (add-hook 'after-revert-hook #'ws-butler-after-save t t)
+        (add-hook 'edit-server-done-hook #'ws-butler-before-save t t))
+    (remove-hook 'after-change-functions #'ws-butler-after-change t)
+    (remove-hook 'before-save-hook #'ws-butler-before-save t)
+    (remove-hook 'after-save-hook #'ws-butler-after-save t)
+    (remove-hook 'before-revert-hook #'ws-butler-before-revert t)
+    (remove-hook 'after-revert-hook #'ws-butler-after-save t)
+    (remove-hook 'edit-server-done-hook #'ws-butler-before-save t)))
 
 ;;;###autoload
 (define-globalized-minor-mode ws-butler-global-mode ws-butler-mode
   (lambda ()
+    "Enable `ws-butler-mode' unless current major mode is exempt."
     (unless (apply #'derived-mode-p ws-butler-global-exempt-modes)
       (ws-butler-mode))))
 
